#include "stdafx.h"
#include "cehttp.h"
#include "MyConf.h"

void MyConf::ReadNumericValue(WCHAR* tszKey, int* pnValue, int nDefaultValue)
{
	DWORD dwRead;
	ReadNumericValue(tszKey, &dwRead, (DWORD)nDefaultValue);
	*pnValue = dwRead;
}

// ------------------------------------------------------------------
MyConf::MyConf()
{
	char	szPathBuf[MAX_PATH];
	int		nLen;

	// The port no listens to
	ReadNumericValue(MYCONF_WSTR_KEY_PORT, &m_nListenPort, MYCONF_NUM_PORT);

	// The number of milliseconds for timeout from accept to data sent.
	ReadNumericValue(MYCONF_WSTR_KEY_TIMEOUT, &m_nTimeOut, MYCONF_NUM_TIMEOUT);
	
	// Keep-Alive flag.
	ReadNumericValue(MYCONF_WSTR_KEY_KEEPALIVE, &m_nKeepAlive, MYCONF_NUM_KEEPALIVE);

	// Start servers
	ReadNumericValue(MYCONF_WSTR_KEY_SERVERS, &m_nStartServers, MYCONF_NUM_SERVERS);

	nLen = ReadStringValue(MYCONF_WSTR_KEY_SRVNAME, szPathBuf, MAX_PATH, MYCONF_STR_SRVNAME);
	m_pszServerSoftware = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszServerSoftware, szPathBuf );

	nLen = ReadStringValue(MYCONF_WSTR_KEY_DOCROOT, szPathBuf, MAX_PATH, MYCONF_STR_DOCROOT);
	m_pszDocRootName = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszDocRootName, szPathBuf );

	nLen = ReadStringValue(MYCONF_WSTR_KEY_CGIROOT, szPathBuf, MAX_PATH, MYCONF_STR_CGIROOT);
	m_pszCgiRootName = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszCgiRootName, szPathBuf );

	nLen = ReadStringValue(MYCONF_WSTR_KEY_DEFPAGE, szPathBuf, MAX_PATH, MYCONF_STR_DEFPAGE);
	m_pszDefaultDocName = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszDefaultDocName, szPathBuf );

	nLen = ReadStringValue(MYCONF_WSTR_KEY_BAUTHFILE, szPathBuf, MAX_PATH, MYCONF_STR_BAUTHFILE);
	m_pszAccessFileName = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszAccessFileName, szPathBuf );

	// logfiles
	nLen = ReadStringValue(MYCONF_WSTR_KEY_LOGFILE, szPathBuf, MAX_PATH, MYCONF_STR_LOGFILE);
	m_pszLogFile = (char*)MALLOC( sizeof(char) * (nLen+1) );
	strcpy( m_pszLogFile, szPathBuf );

	ReadNumericValue(MYCONF_WSTR_KEY_LOGSIZE, &m_nLogSize, MYCONF_NUM_LOGSIZE);

	ReadNumericValue(MYCONF_WSTR_KEY_DNSLOOKUP, &m_nDNSLookup, MYCONF_NUM_DNSLOOKUP);

}

// ------------------------------------------------------------------
MyConf::~MyConf()
{
	FREEIF(m_pszServerSoftware);
	FREEIF(m_pszDocRootName);
	FREEIF(m_pszCgiRootName);
	FREEIF(m_pszDefaultDocName);
	FREEIF(m_pszAccessFileName);
	FREEIF(m_pszLogFile);

}

// ------------------------------------------------------------------
LONG MyConf::ReadTStringValue(WCHAR* tszKey, WCHAR* tszValue, int nValueSize)
{
	HKEY	hKey;
	DWORD	dwType	= REG_SZ;
	LONG	lErr;

	memset(tszValue, 0, nValueSize);

	lErr = RegOpenKeyEx(HKEY_LOCAL_MACHINE, MYCONF_WSTR_SUBKEY, 0, KEY_QUERY_VALUE, &hKey);
	if( lErr != ERROR_SUCCESS )
	{
		return lErr;
	}

	lErr = RegQueryValueEx(hKey, tszKey, NULL, &dwType, (LPBYTE)tszValue, (DWORD*)&nValueSize);

	if( lErr != ERROR_SUCCESS )
	{
		return lErr;
	}
		
	RegCloseKey(hKey);
	return ERROR_SUCCESS;
}

// ------------------------------------------------------------------
int MyConf::ReadStringValue(WCHAR* tszKey, char* szValue, int nValueSize, char* szDefaultValue)
{
	LONG	lErr;
	DWORD	dwValueSize = sizeof(WCHAR) * nValueSize;
	WCHAR*	ptszValue = (WCHAR*)MALLOC( dwValueSize );

	memset( ptszValue, 0, nValueSize );
	lErr = ReadTStringValue(tszKey, ptszValue, dwValueSize-sizeof(WCHAR));

	if( lErr != ERROR_SUCCESS )
	{
		strncpy(szValue, szDefaultValue, nValueSize);
		FREE( ptszValue );
		return strlen(szValue);
	}

	if( 0 == WideCharToMultiByte(CP_ACP, 0, ptszValue, -1, szValue, nValueSize, NULL, NULL) )
	{
		if( szDefaultValue )
			strcpy(szValue, szDefaultValue);
		else
			szValue[0] = '\0';
	}
	FREEIF( ptszValue );

	return strlen(szValue);
}

// ------------------------------------------------------------------
void MyConf::ReadNumericValue(WCHAR* tszKey, DWORD* pdwValue, DWORD dwDefaultValue)
{
	HKEY	hKey;
	DWORD	dwType	= REG_DWORD;
	LONG	lErr;

	lErr = RegOpenKeyEx(HKEY_LOCAL_MACHINE, MYCONF_WSTR_SUBKEY, 0, KEY_QUERY_VALUE, &hKey);
	if( lErr != ERROR_SUCCESS )
	{
		*pdwValue = dwDefaultValue;
		RegCloseKey(hKey);
		return;
	}

	DWORD	dwValueSize = sizeof(DWORD);
	
	lErr = RegQueryValueEx(hKey, tszKey, NULL, &dwType, (LPBYTE)pdwValue, &dwValueSize);

	if( lErr != ERROR_SUCCESS )
	{
		*pdwValue = dwDefaultValue;
		RegCloseKey(hKey);
		return;
	}

	RegCloseKey(hKey);
}

// ------------------------------------------------------------------
BOOL MyConf::WriteStringValue(WCHAR* tszKey, WCHAR* tszValue)
{
	HKEY	hKey;
	DWORD	dwRet	= 0;
	LONG	lErr;
	
	INT		nULen;
	lErr = RegCreateKeyEx(HKEY_LOCAL_MACHINE, MYCONF_WSTR_SUBKEY, 0, TEXT(""), 0, KEY_ALL_ACCESS, NULL, &hKey, &dwRet);
	if( lErr != ERROR_SUCCESS )
	{
		goto ERR_RETURN;
	}

	nULen = wcslen(tszValue) * sizeof(WCHAR);
	lErr = RegSetValueEx(hKey, tszKey, 0, REG_SZ, (LPBYTE)tszValue, nULen);
	if( lErr != ERROR_SUCCESS )
	{
		goto ERR_RETURN;
	}
	return TRUE;

ERR_RETURN:
	RegCloseKey(hKey);
	return FALSE;
}

// ------------------------------------------------------------------
BOOL MyConf::WriteStringValue(WCHAR* tszKey, char* szValue)
{
	int		nLen;
	BOOL	nRet;
	WCHAR*	p;

	nLen = MultiByteToWideChar( CP_ACP, 0, szValue, -1, NULL, 0 );

	p = (WCHAR*)MALLOC( sizeof( WCHAR ) * (nLen+1) );
	memset( p, 0, (nLen+1)*sizeof(WCHAR) );

	if( 0 != MultiByteToWideChar(CP_ACP, 0, szValue, -1, p, nLen+1 ) )
	{
		nRet = WriteStringValue(tszKey, p);
	}
	else
	{
		nRet = FALSE;
	}

	FREEIF( p );
	return nRet;

}

// ------------------------------------------------------------------
BOOL MyConf::WriteIntValue(WCHAR* tszKey, int nNumber)
{
	HKEY	hKey;
	DWORD	dwRet	= 0;
	LONG	lErr;
	DWORD	dwValue = nNumber;

	lErr = RegCreateKeyEx(HKEY_LOCAL_MACHINE, MYCONF_WSTR_SUBKEY, 0, TEXT(""), 0, KEY_ALL_ACCESS, NULL, &hKey, &dwRet);
	if( lErr != ERROR_SUCCESS )
	{
		goto ERR_RETURN;
	}

	lErr = RegSetValueEx(hKey, tszKey, 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(DWORD));
	if( lErr != ERROR_SUCCESS )
	{
		goto ERR_RETURN;
	}
	return TRUE;

ERR_RETURN:
	RegCloseKey(hKey);
	return FALSE;

}