#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include "str.h"
#include "cehttp.h"
#include "httphelper.h"
#include "MyVarMem.h"
#include "MyHash.h"
#include "MyHttpSock.h"
#include "MyTime.h"
#include "MyErrPage.h"



#if !defined(_WIN32_WCE)
#include <direct.h>
#endif

#include "resource.h"

#define	HEADER_LINE_MAX	512		// HeaderLine's max size
#define HEADER_SiZE_MAX	4096	// Header max size is 4KB
// temporary buffers
#define SOCKET_HEAD_BUF	1500	// Size of head socket buffer
#define SOCKET_BODY_BUF	1500	//     ... it size is almost the same as MTU.

// strings
#define	STR_KEEP_ALIVE		"Keep-Alive"
#define STR_CLOSE			"Close"

#define RPIPE	0
#define WPIPE	1

//-----------------------------------------------
// For saving time, No. 0,2,5 is fixed
#define HTTPSTATUS_200	0
#define HTTPSTATUS_302	2
#define HTTPSTATUS_401	5

HTTP_STATUS	hsStatus[] = 
{
	{200, "200 OK"								},	// 0
	{301, "301 Moved Permanently"				},
	{302, "302 Object Moved"					},	// 2
	{304, "304 Not Modified"					},
	{400, "400 Bad Request"						},
	{401, "401 Unauthorized"					},	// 5
	{403, "403 Access Forbidden"				},
	{404, "404 Object Not Found"				},
	{411, "411 Length Required"					},
	{413, "413 Request Entity Too Large"		},
	{500, "500 Internal Server Error"			},
	{501, "501 Not Implemented"					},
	// {503, "503 Server busy"					},	// this response processed by parent thread.
	{999, "999 CGI"								},	// CEHTTP unique
	{000, "500 Internal Server Error"			}	// terminater
};


//-----------------------------------------------
// content_type

MyContentType		g_ctMime;			// MIMETYPE list
MyErrPage			g_cfErrHTML;		// ERRORHTML list

typedef int (*CEHTTP_CHX_EXTENTION)(MyHash*, MyVarMem*, MyVarMem*);
typedef int (*CEHTTP_DLL_EXTENTION)(const void*, int, const void*, int, const void*, int, void**);

static int ParseString( const char* pReq, char* szReqKey, char* szReqValue );
static int ParseRequestLine( const char* pReq, char* szCmd, char* szPath, char* szHttpVer );
static int AbsoluteURL2relativityURL(char* pszAbsolutePath);
static int DirectoryAccessCheck( const char* pszRelativePath );


MyHttpSock::MyHttpSock()
{
	m_dwRecvTimeOut = 30000;
}

MyHttpSock::MyHttpSock(SOCKET s) : MySock(s)
{
	m_dwRecvTimeOut = 30000;
}

MyHttpSock::~MyHttpSock()
{
	FreeMemory();
}

//-----------------------------------------------------------------------------
//	@detail
//		Recv data from a client.
//		Investigate the HTTP request, and decide a status.
//	@param
//		int* pbClose	Not to keep alive flag
//	@return
//		0 indicate socket error.
//		the other, HTTP status.
//-----------------------------------------------------------------------------
int MyHttpSock::RecvRequest(int* pbClose)
{
	char szHeadBuf[SOCKET_HEAD_BUF];
	m_hReqHead.Clear();
	m_hReqBody.Clear();
	timeval ptvTimeOut;
	char* pReq = NULL;
	strcpy(m_hrHttpReq.szHttpVer, "HTTP/1.0");

	GetTimeOut(&ptvTimeOut);

	//-------------------------------------------
	// Recive
	//-------------------------------------------

	// Get http header
	DWORD	dwStartTime = GetTickCount();
	MyVarMem hReq;
	int nRecvLen;

	while(1)
	{
		char*	p;
		memset( szHeadBuf, 0, SOCKET_HEAD_BUF );
		nRecvLen = RecvData( szHeadBuf, SOCKET_HEAD_BUF );

		if( nRecvLen == SOCKET_ERROR )		//*** socket error
		{
			Close();
			return 0;
		}
		else if( nRecvLen > 0 )				//*** recive
		{
			// RFC2626 sec.5
			if( 0 == hReq.Add((void*)szHeadBuf, nRecvLen) )
			{
				return 500;
			}

			if( hReq.GetLength() > HEADER_SiZE_MAX )
			{
				return 400;		// Bad Request
			}

			// Judge the end of header
			const char* pszReqHeadBuf = (const char*)hReq.GetBuffer();

			if( p = strstr(pszReqHeadBuf, "\r\n\r\n") )
			{
				p += 4;
			}
			else if( p = strstr(pszReqHeadBuf, "\n\n") )
			{
				p += 2;
			}

			// here, at the end of header.
			if( p != NULL )
			{
				// Add read header block to m_ReqHead.
				int nHeaderLen = (p - pszReqHeadBuf);
				if( 0 == m_hReqHead.Add(hReq.GetBuffer(), nHeaderLen) )
					return 413;
				
				// A part to have over taken is added to a m_hReqBody.
				int nDrop = hReq.GetLength() - nHeaderLen;
				if( nDrop > 0 )
				{
					if( 0 == m_hReqBody.Add( p, nDrop ) )
						return 500;
				}

				hReq.Clear();

				//-------------------------------------------------------------
				// Safety check of header block
				// each length of headers must be under HEADER_LINE_MAX
				//-------------------------------------------------------------
				pReq = (char*)(const char*)m_hReqHead;
	
				for( int count = 0; *pReq; pReq++, count++ )
				{
					if( count > HEADER_LINE_MAX )
						return 400;	// Bad Request

					if( *pReq == '\r' )
					{
						count = 0;

						if( pReq[1] == '\0' )
							return 400;		// Bad Request
						
						pReq += 2;
						continue;
					}
				}

				// Parse request line.
				int ret;
				ret = ReadRequestLine((const char*)m_hReqHead);
				if( ret != 200 ) return ret;

				// Parse request header
				ret = ReadRequestHeaders((const char*)m_hReqHead);
				if( ret != 200 ) return ret;

				// DNS lookup
				if( m_pServerInfo->nDNSLookup )
				{
					sockaddr_in* psa = m_hrHttpReq.psaRemoteHost;
					u_char* p = (u_char*)&(psa->sin_addr.S_un.S_un_b);
					HOSTENT* phe = gethostbyaddr( (const char*)p, 4, AF_INET ); //IPv4
					
					if( NULL != phe && NULL != phe->h_name )
					{
						m_hashReqHeader.Set( LOG_REMOTE_HOST,phe->h_name );
					}
				}

				// HTTP/1.1 must need Host: header
				// @RFC2616 sec.19.6.1.1
				if( !strcmp( m_hrHttpReq.szHttpVer, "HTTP/1.1" ) &&
					!m_hashReqHeader.GetValueI("host") )
					return 400;	// Bad Request

				break;
			}
		}
		else // timeout
		{
		}


		// timeout check.
		DWORD dwSlipTime = (GetTickCount() - dwStartTime);
		
		// max 90sec. 47 days problem.
		if(	dwSlipTime > m_dwRecvTimeOut || dwSlipTime > 90000 )
		{
			//m_bKeepAlive = FALSE;
			return 0; // Time-out
		}

		// Check stop the keep-alive.
		if( hReq.GetLength() == 0 && *pbClose )
		{
			return 0; // Time-out
		}
	}	// end of while


	//-------------------------------------------
	// Make file path for windows system
	//    MakePath is convert szFilPath to absolute path
	//    # e.g. \image\icon.gif --> \wwwroot\image\icon.gif
	//    And make the other members of m_hrHttpReq
	//-------------------------------------------
	int status = MakePath();

	// --------------------------------
	// If the request is POST, have to work for a while the back.
	if( m_hrHttpReq.HttpCmd == HTTPPOST )
	{
		char* p = m_hashReqHeader.GetValueI("content-length");
		
		if( p == NULL )
			return 411; // Length Required

		long lTotalSize = m_hReqBody.GetLength();
		long lContentLength = atol(p);

		if( lContentLength < 0 )
		{
			return 400;	// Bad Request
		}

		nRecvLen = 0;

		while(1)
		{
			// size check.
			lTotalSize += nRecvLen;

			if( lTotalSize >= lContentLength )
				return status;

			nRecvLen = RecvData(szHeadBuf, SOCKET_HEAD_BUF);

			// Socket error || time out
			if( nRecvLen == SOCKET_ERROR || nRecvLen == 0)
			{
				m_hReqBody.Clear();
				return 0;
			}

			// Store posted data to the buffer
			if( 0 == m_hReqBody.Add((void*)szHeadBuf, nRecvLen) )
			{
				m_hReqBody.Clear();
				return 413; // Request Entity Too Large
			}

			// Total timeout check.
			if( GetTickCount() - dwStartTime > m_dwRecvTimeOut )
			{
				m_hReqBody.Clear();
				return 0; // Time-out
			}
		}
	}

	return status;
}

//-----------------------------------------------------------------------------
//	@detail
//		Main response function
//	@param
//		nHttpStatus		Result of http status
//		int bClose		Keep-Alive flag
//								default is 0
//	@return
//		0 indicate socket error.
//		other indicate http status.
//-----------------------------------------------------------------------------
int MyHttpSock::SendResponse(int nHttpStatus, int bClose)
{
	int	nSockStatus = 0;

	// According to http status, send default message.

	// ====================================================================
	// BASIC AUTH
	// ====================================================================
	nHttpStatus = CheckBasicAuth( nHttpStatus, m_hrHttpReq.szFilePath, m_pServerInfo->pszAccessFileName );
	
	switch( nHttpStatus )
	{
		// ----------------------------------
		// ERROR
		// ----------------------------------
		case 0:
			return 0;
		// --------------------------------------------------------------------
		// 200 OK
		case 200:
			return SendHttpFile(nHttpStatus, bClose);

		// --------------------------------------------------------------------
		// 401 Access Denied
		case 401:
			return Send401Response(nHttpStatus, bClose);

		// --------------------------------------------------------------------
		// 999 CehttpGatewayInterface
		case 999:
			return Send999Response(nHttpStatus, bClose);

		default:
			return SendDefaultHttpResponse(nHttpStatus, bClose, __LINE__);
	}

	if( nSockStatus < 1 )
	{
		return 0;
	}
	return nHttpStatus;
}

//-----------------------------------------------------------------------------
//	@detail
//		Send a responce according to HTTP status.
//	@param
//		nHttpStatus		HTTP status
//		int bClose		Keep alive flag. default is true.
//	@return
//		0 indicate SOCKET_ERROR
//		the other, HTTP status
//	@RFC2616
//-----------------------------------------------------------------------------
int MyHttpSock::SendDefaultHttpResponse(int nHttpStatus, int bClose, int nAddtionalInfo/*=0*/)
{
	char szHeadBuf[256];

	int ii = 0;
	int nSockStatus = 0;
	char* pszErrorMsgHtml = NULL;

	// Serch error page to fit http sataus.
	const char* pszErrorMsgFile = g_cfErrHTML.GetErrorPage( nHttpStatus );

	if( pszErrorMsgFile )
	{
		strcpy( m_hrHttpReq.szUrlPath, pszErrorMsgFile );
		switch( MakePath() )
		{
		case 200:
			return SendHttpFile( nHttpStatus, TRUE );
		case 999:
			return Send999Response( 200, TRUE );
		default:
			break;
		}
	}

	// --------------------------------------------------------------
	// Send HTTP message according to nHttpStatus number.
	while(hsStatus[ii].nStatus != 0)
	{
		if( hsStatus[ii].nStatus == nHttpStatus )
		{
			sprintf(szHeadBuf,
				"%s %s(%d)\r\n"
				"Server: %s\r\n"
				"Connection: %s\r\n"		// @RFC2616 sec.14.9
				"\r\n"

				, m_hrHttpReq.szHttpVer, hsStatus[ii].pszString, nAddtionalInfo
				, m_pServerInfo->pszSoftware
				, (bClose ? STR_CLOSE : STR_KEEP_ALIVE)
				);

			nSockStatus = SendData(szHeadBuf);

			if( nSockStatus < 1 )
			{
				return 0;
			}
			
			return nHttpStatus;
		}
		ii++;
	}

	if( hsStatus[ii].nStatus == 0 )
	{
		sprintf( szHeadBuf,
			"%s %d\r\n"
			"Connection: Close\r\n"
			"\r\n"
			, m_hrHttpReq.szHttpVer, nHttpStatus, nAddtionalInfo
			);

		if( SendData(szHeadBuf) < 1 )
		{
			return 0;
		}
	}

	return nHttpStatus;

}

//-----------------------------------------------------------------------------
//	@detail
//		file transfer response function.
//
//		Existence of a file is not checked.
//		It is assumed that it is certainly there.
//
//	@param
//		nHttpStatus		Result of http status
//		int bClose		Keep-Alive flag
//								default is 0
//	@return
//		0 indicate socket error.
//		other indicate http status.
//-----------------------------------------------------------------------------
int MyHttpSock::SendHttpFile(int nHttpStatus, int bClose)
{
	WCHAR	wszFile[MAX_PATH];
	//MakeEnvironment();

	char szHeadBuf[SOCKET_HEAD_BUF];
	char szBodyBuf[SOCKET_BODY_BUF];

	*szHeadBuf = '\0';
	*szBodyBuf = '\0';

	char* pszFPath = m_hrHttpReq.szFilePath;
	int nFileNameLen = strlen( m_hrHttpReq.szFilePath );

	// Check extend word
	char* pszExtend = &m_hrHttpReq.szFilePath[nFileNameLen-1];

	while( *pszExtend != '\\' && *pszExtend != '.' )
	{
		pszExtend--;
	}

	// Check mime type
	char* pszMimeType = g_ctMime.GetMimeType(pszExtend);

	// --------------------------------------------------------------
	// MOD extension
	// if the first word is \ that is server page.
	// --------------------------------------------------------------
	if( *pszMimeType == '\\' ) 
	{
		MakeEnvironment();
		// --------------------------
		// SCRIPT_NAME
		// --------------------------
		int nDRootLen = strlen( m_pServerInfo->pszDocRootName );
		m_hashEnv.Set( "SCRIPT_NAME", &pszFPath[nDRootLen] );
		bslash2slash( m_hashEnv.GetValue("SCRIPT_NAME") );

		return ExecuteChx( pszMimeType, bClose );
	}

	// trancelate filename to unicode
	if( 0 == MultiByteToWideChar(CP_ACP, 0, pszFPath, -1, wszFile, MAX_PATH) )
	{
		return SendDefaultHttpResponse(500, bClose, __LINE__);
	}

	// ------------------------------
	// HTTP/1.x OK xxxx
	// Server: xxxxx
	// Content-Length: xxxx
	// Last-Modified: Mon, 24 Sep 2001 15:20:34 GMT
	// Content-Type: xxxx/xxxx
	// ------------------------------
	MyTime time(m_hrHttpReq.wfdFile.ftLastWriteTime);

	// --------------------------------------------------------------
	// Send HTTP message according to nHttpStatus number.
	int ii = 0;
	while(hsStatus[ii].nStatus != 0)
	{
		if( hsStatus[ii].nStatus == nHttpStatus )
		{
			sprintf(szHeadBuf,
				"%s %s\r\n"
				"Server: %s\r\n"
				"Connection: %s\r\n"		// @RFC2616 sec.14.9
				"Content-Length: %u\r\n"
				"Last-Modified: %s\r\n"
				"Content-Type: %s\r\n"
				"\r\n"

				,m_hrHttpReq.szHttpVer, hsStatus[ii].pszString
				, m_pServerInfo->pszSoftware
				, (bClose ? STR_CLOSE : STR_KEEP_ALIVE)
				, m_hrHttpReq.wfdFile.nFileSizeLow
				, time.GetFormatRfc1123()
				, pszMimeType
				);
			break;
		}
		ii++;
	}

	if( hsStatus[ii].nStatus == 0 )
	{
		sprintf( szHeadBuf, 
			"%s %d\r\n"
			"\r\n"
			, m_hrHttpReq.szHttpVer, nHttpStatus
			);
	}
	// ------------------------------
	// Send file
	// ------------------------------

	DWORD	dwReadBytes;

	// HEAD command doesn't send file
	if( m_hrHttpReq.HttpCmd == HTTPGET )
	{
		// Take file size for log.
		sprintf( szBodyBuf, "%d", m_hrHttpReq.wfdFile.nFileSizeLow );
		m_hashReqHeader.Set( LOG_SEND_DATA, szBodyBuf );

		// file open
		HANDLE hFile = CreateFile( wszFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

		if( hFile == INVALID_HANDLE_VALUE )
		{
			return SendDefaultHttpResponse( 500, bClose, __LINE__ );
		}


		if(  SendData( szHeadBuf ) < 1 )
		{
			CloseHandle( hFile );
			return 0;
		}
		
		do
		{
			// the file is divided and transmitted.
			if( ReadFile( hFile, szBodyBuf, SOCKET_BODY_BUF, &dwReadBytes, NULL ) )
			{
				if( dwReadBytes == 0 )
				{
					break;
				}

				if(  SendData(szBodyBuf, dwReadBytes) < 1 )
				{
					CloseHandle( hFile );
					return 0;
				}
			}
			else
			{
				break;
			}
		}while( dwReadBytes > 0 );

		CloseHandle( hFile );
	}
	else if( m_hrHttpReq.HttpCmd == HTTPHEAD )
	{
		if(  SendData( szHeadBuf ) < 1 )
		return 0;
	}

	return nHttpStatus;
}

//-----------------------------------------------------------------------------
//	@note:
//		InternetExplorer is influenced by 'Content-Type' though server says 
//		'not modified'.
//		For example, when GIF file is requested and then server say
//			"HTTP/1.1 304" "content-type: text/html" "not modified."
//		IE is try to show GIF file as text/html.
//		So, server must not say about content-type when response is 304.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	@detail
//		401 Unauthorized
//	@param
//		nHttpStatus		Result of http status
//		int bClose		Keep-Alive flag
//								default is 0
//	@return
//		0 indicate socket error.
//		other indicate http status.
//-----------------------------------------------------------------------------
int MyHttpSock::Send401Response(int nHttpStatus, int bClose)
{
	char szHeadBuf[512];

	// ----------------------------------
	// HTTP/1.1 401 Access Denied
	// Server: xxxxx
	// WWW-Authenticate: Basic realm="xxxx"
	// Content-Length: nnnn
	// Content-Type: text/html
	// ----------------------------------
	char* pszHost;
	pszHost = ( m_hashReqHeader.GetValueI("host") );
	if( !pszHost )
		pszHost = "localhost";

	sprintf(szHeadBuf, 
		"%s %s\r\n"
		"Server: %s\r\n"
		"WWW-Authenticate: Basic realm=\"%s%s\"\r\n"
		"Connection: close\r\n"
		"\r\n"
		
		,m_hrHttpReq.szHttpVer, hsStatus[HTTPSTATUS_401].pszString
		,(char*)m_pServerInfo->pszSoftware
		,  pszHost, m_hrHttpReq.szUrlPath

	);

	if( SendData(szHeadBuf) < 1 ) return 0;
	return nHttpStatus;
}

//-----------------------------------------------------------------------------
//	@detail
//		999 response function (Scripts execute)
//	@param
//		nHttpStatus		Result of http status
//		int bClose		Keep-Alive flag
//								default is 0
//	@return
//		0 indicate socket error.
//		other indicate http status.
//-----------------------------------------------------------------------------
int MyHttpSock::Send999Response(int nHttpStatus, int bClose)
{
	MakeEnvironment();
	char* pszFPath = m_hrHttpReq.szFilePath;
	int nFPathLen = strlen(pszFPath);
	
	// --------------------------
	// SCRIPT_NAME
	// --------------------------
	int nDRootLen = strlen(m_pServerInfo->pszDocRootName);
	m_hashEnv.Set("SCRIPT_NAME", &pszFPath[nDRootLen]);
	bslash2slash(m_hashEnv.GetValue("SCRIPT_NAME"));

	// ----------------------------------------------------------------
	// Call CHX extension.
	if( 0 == STRNICMP( &pszFPath[nFPathLen] - 4, ".chx", 4 ) )
	{
		return ExecuteChx( pszFPath, bClose );
	}
	// ----------------------------------------------------------------
	// Call DLL extension.
	else if( 0 == STRNICMP( &pszFPath[nFPathLen] - 4, ".dll", 4 ) )
	{
		return ExecuteDll( pszFPath, bClose );
	}
	// ----------------------------------------------------------------
	// Call CGI process.
#if !defined(_WIN32_WCE)
	else if( 0 == STRNICMP( &pszFPath[nFPathLen] - 4, ".exe", 4 ) )
	{
		return ExecuteCGI( pszFPath, bClose );
	}
#endif
	// ----------------------------------------------------------------
	// the other
	else
	{
		//403 Access Forbidden
		return SendDefaultHttpResponse(403, bClose, __LINE__);
	}
}


//-----------------------------------------------------------------------------
//	@detail
//		Parse request line
//		
//		Make 3 request header like as..
//			GET /image/icon.gif HTTP/1.0
//
//		And make QUERY_STRING in 'm_hashReqHeader', if there.
//
//		And make file information.
//				m_hrHttpReq.HttpCmd
//				m_hrHttpReq.szUrlPath
//				m_hrHttpReq.szFilePath
//
//	@param
//		
//	@return
//		400 Bad reuest
//		200 Successful
//		500 InternalServerError
//		501 not implement
//
//	RFC2616 sec.5.1
//		Request-Line   = Method SP Request-URI SP HTTP-Version CRLF
//-----------------------------------------------------------------------------
int MyHttpSock::ReadRequestLine(const char* pSrc)
{
	char szTmp1[HEADER_LINE_MAX] = {0};
	char szTmp2[HEADER_LINE_MAX] = {0};
	char szTmp3[HEADER_LINE_MAX] = {0};

	//-------------------------------------------
	// Extract request line from headers.
	//-------------------------------------------
	
	const char* ps = pSrc;
	char* pd = szTmp1;
	int ii = 0;

	for( ii = 0; *ps != '\r' && *ps != '\n' && *ps != '\0' && ii < HEADER_LINE_MAX-1; ii++, pd++, ps++)
	{
		*pd = *ps;
	}
	*pd = '\0';

	m_hashReqHeader.Set( LOG_REQUEST_LINE, szTmp1);


	//-------------------------------------------
	// Analyze request line
	//-------------------------------------------
	char* pReq = (char*)pSrc;
	
	if( 3 > ParseRequestLine(pReq, szTmp1, szTmp2, szTmp3) )
		return 400;	// Bad Request


	// Inquire HTTP command
	if( !strcmp( szTmp1, "GET" ) )
	{
		m_hrHttpReq.HttpCmd = HTTPGET;
	}
	else if( !strcmp( szTmp1, "HEAD" ) )
	{
		m_hrHttpReq.HttpCmd = HTTPHEAD;
	}
	else if( !strcmp( szTmp1, "POST" ) )
	{
		m_hrHttpReq.HttpCmd = HTTPPOST;
	}
	else if( 
		!strcmp( szTmp1, "PUT" )	||
		!strcmp( szTmp1, "DELETE" )	||
		!strcmp( szTmp1, "TRACE" )	||
		!strcmp( szTmp1, "OPTIONS" )||
		!strcmp( szTmp1, "CONNECT" )||
		!strcmp( szTmp1, "LINK" )	||
		!strcmp( szTmp1, "UNLINK" )	)
	{
		m_hrHttpReq.HttpCmd = HTTPNOTIMPLEMENTED;
		return 501;	// Not Implemented
	}
	else
	{
		m_hrHttpReq.HttpCmd = HTTPUNKNOWN;
		return 400;	// Bad Request
	}


	//-------------------------------------------
	// Analyze HTTPversion
	//-------------------------------------------
	if( !strcmp( szTmp3, "HTTP/1.0" ) || !strcmp( szTmp3, "HTTP/1.1" ) || !strcmp( szTmp3, "HTTP/0.9" ) )
	{
		strcpy(m_hrHttpReq.szHttpVer, szTmp3);
	}
	else
	{
		// If version is unknown,
		strcpy(m_hrHttpReq.szHttpVer, "HTTP/1.0");
		return 400;	// Bad Request
	}

	//-------------------------------------------
	// Analyze URL path
	//-------------------------------------------
	char* pszPathFrom = szTmp2;
	char* pszUrlPath  = m_hrHttpReq.szUrlPath;

	// When the URI is http://servername/..../,
	// @RFC2616 sec.5.1.2 absoluteURI
	// servername is assumed correctly. it is ignored.

	while( *pszPathFrom != '\0' ) {*pszUrlPath++ = *pszPathFrom++;}
	*pszUrlPath = '\0';

	return 200;

}

//-----------------------------------------------------------------------------
//	@detail
//		The request header string is dvided and put it into m_hashReqHeader
//	@param
//		pSrc	Request data from browser.
//	@return
//		400 Bad request
//		200 OK
//		500 InternalServerError
//
//	@RFC2616 sec.5
//-----------------------------------------------------------------------------
int MyHttpSock::ReadRequestHeaders(const char* pSrc)
{
	//-------------------------------------------
	// Analyze Request header.
	//-------------------------------------------
	char szReqKey[HEADER_LINE_MAX], szReqValue[HEADER_LINE_MAX];

	char* pReq = (char*)pSrc;

	// Go the next line to avoid request line.(like GET /index.html HTTP/1.0 ...)
	if( NULL == ( pReq = strstr( pReq, "\r\n" )) )
	{
		return 400;	// Bad Request
	}
	pReq += 2;
	
	while( *pReq != '\0' )
	{
		if( *pReq == '\r' )	// end of request header
			break;
		
		if( 2 == ParseString( (const char*)pReq, szReqKey, szReqValue ) )
		{
			// Request header must under 4000 bytes
			if( !m_hashReqHeader.Set(szReqKey, szReqValue) ||
				m_hashReqHeader.GetLength() > 4000)
			return 500;
		}

		if( NULL == (pReq = strstr( pReq, "\r\n" )) )
			return 400;	// Bad Request

		pReq += 2;
	}

	return 200;
}

//-----------------------------------------------------------------------------
//	@detail
//		Make a file path.
//
//	@param
//	@return
//		999		When the request call CGI, return 999.
//		404
//		400
//		302
//		200
//	@in
//		SET m_hrHttpReq.szUrlPath
//
//	@out
//		m_hrHttpReq
//
//	@note
//		It was restricted that the back directory symbol '..' like following.
//			../<cgiroot>/cgi.cgi
//-----------------------------------------------------------------------------
int MyHttpSock::MakePath()
{
	// ----------------------------------------------------
	// When the URI is http://<server>:<port>/..../,
	// Truncate http://<server>:<port>
	// @RFC2616 sec.5.1.2 absoluteURI
	// CEHTTP assumed that servername is correctly.
	// ----------------------------------------------------
	if( !AbsoluteURL2relativityURL( m_hrHttpReq.szUrlPath ) )
		return 400;	// Bad Request

	// ----------------------------------------------------
	// QUERY_STRING
	// ----------------------------------------------------
	char* pQS =  m_hrHttpReq.szUrlPath;
	while( *pQS != '?' && *pQS != '\0' ) pQS++;
	
	if( *pQS == '?' )
	{
		*pQS = '\0';
		pQS++;
		m_hashReqHeader.Set("QUERY_STRING", pQS);
	}
	
	// prepare to make file path, copy URL path to File path
	strcpy( m_hrHttpReq.szFilePath, m_hrHttpReq.szUrlPath );
	slash2bslash( m_hrHttpReq.szFilePath );

	
	int	bIsCgi = FALSE;
	char szFullPath[MAX_PATH] = {0};// work
	int	nFullPathLen;				// work
	int nDRootLen  = strlen( m_pServerInfo->pszDocRootName );
	int nCRootLen  = strlen( m_pServerInfo->pszCgiRootName );
	int nDefDocLen = strlen( m_pServerInfo->pszDefaultDocName );
	int nSrcLen    = strlen( m_hrHttpReq.szFilePath );

	memset( &m_hrHttpReq.wfdFile, 0, sizeof(WIN32_FIND_DATA) );

	// Simple size check.
	if( nDRootLen + nSrcLen + nDefDocLen + 1 > MAX_PATH-1 )
		return 400;	// Bad Request

	
	// ----------------------------------------------------
	// Simple access check for directory
	// ----------------------------------------------------
	if( !DirectoryAccessCheck( m_hrHttpReq.szFilePath ) )
	{
		return 400;	// Bad request.
	}

	// ----------------------------------------------------
	// Join document root and path
	// ----------------------------------------------------
	szFullPath[0] = '\0';

	strcat( szFullPath, m_pServerInfo->pszDocRootName );
	strcat( szFullPath, m_hrHttpReq.szFilePath );

	slash2bslash( szFullPath );

	nFullPathLen = strlen( szFullPath );

	// convert to file path is finish.......

	// ----------------------------------------------------
	// To determine the request is Which CGI file or not,
	// compare the request and CGI root directory.
	// ----------------------------------------------------
	if( 0 == STRNICMP( szFullPath, m_pServerInfo->pszCgiRootName, 
									strlen(m_pServerInfo->pszCgiRootName) ) )
	{
		bIsCgi = TRUE;
	}

	// ----------------------------------------------------
	// When the last word is \, add a default page strings.
	// ----------------------------------------------------
	if( szFullPath[nFullPathLen-1] == '\\' )
	{
		if( !bIsCgi )
		{
			// If the last charcter is '\', add string 'index.html'(pszDefaultDocName).
			strcat( szFullPath, m_pServerInfo->pszDefaultDocName );
		}
	}

	// Serch file.
	HANDLE hFFF = FindFirstFileMBCS( szFullPath, &m_hrHttpReq.wfdFile );

	if( INVALID_HANDLE_VALUE != hFFF )
	{
		// ------------------------------------------------
		// file found.
		// ------------------------------------------------
		FindClose( hFFF );
		hFFF = INVALID_HANDLE_VALUE;

		// ------------------------------------------------
		// When it is directory or not file,
		// add string 'index.html'(pszDefaultDocName) and try it.
		// ------------------------------------------------
		if( m_hrHttpReq.wfdFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
		{
			// try default document.
			strcat( szFullPath, "\\" );
			strcat( szFullPath, m_pServerInfo->pszDefaultDocName );

			hFFF = FindFirstFileMBCS( szFullPath, &m_hrHttpReq.wfdFile );

			if( INVALID_HANDLE_VALUE == hFFF )
			{
				return 404;	// "Object Not Found"
			}

			// Found a default document at there.
			FindClose(hFFF);
			hFFF = INVALID_HANDLE_VALUE;
		}
		
		if( bIsCgi )
		{
			// Set default PAHT_INFO & PATH_TRANSLATED
			m_hashEnv.Set( "PATH_INFO",	"" );
			m_hashEnv.Set( "PATH_TRANSLATED", m_pServerInfo->pszDocRootName );
		}
	}
	else
	{
		if( bIsCgi )
		{
			// ------------------------------------------------
			// Work for PATH_INFO
			//  If file path is not exist, do PATH_INFO check.
			// ------------------------------------------------
			// /<cgi root>/.../xxx.cgi/here is the path info/
			//                                            

			char* pTmp = &szFullPath[ nCRootLen + 1 ];
			// \<cgi_path>\sample.cgi\path1\path2\path3
			//            ^-p
			// go to next '\'.
			for( ;*pTmp != '\\' && *pTmp != '\0'; pTmp++ );
			// \<cgi_path>\sample.cgi\path1\path2\path3
			//             ^-p

			while(1)
			{
				if( *pTmp == '\0' )
				{
					return 404;	// "Object Not Found"
				}
				
				*pTmp = '\0';

				hFFF = FindFirstFileMBCS( szFullPath, &m_hrHttpReq.wfdFile );

				// Now found! executable file,
				if(
					( INVALID_HANDLE_VALUE != hFFF )
					&& !( m_hrHttpReq.wfdFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ))
				{
					// CGI call settled, make hashEnv.
					FindClose(hFFF);

					// Add "\" to head of PATH_INFO
					*pTmp = '\\';

					// --------------------------
					// PATH_INFO
					// --------------------------
					m_hashEnv.Set( "PATH_INFO", pTmp );
					bslash2slash( m_hashEnv.GetValue("PATH_INFO") );

					// --------------------------
					// PATH_TRANSLATED
					// --------------------------
					char	path_trans[MAX_PATH];

					if( nDRootLen + strlen( pTmp ) >= MAX_PATH -1 )
						return 400;
					
					strcpy( path_trans, m_pServerInfo->pszDocRootName );
					strcat( path_trans, pTmp );
					m_hashEnv.Set( "PATH_TRANSLATED",	path_trans );
					slash2bslash( m_hashEnv.GetValue( "PATH_TRANSLATED" ) );


					// Separete PATH_INFO again
					*pTmp = '\0';

					break;
					// <-- PATH_INFO process end.
				}

				*pTmp++ = '\\';

				// next
				for( ;*pTmp != '\\' && *pTmp != '\0'; pTmp++ );
			}
		}
		else
		{
			return 404;	// "Object Not Found"
		}
	}
	
	strncpy( m_hrHttpReq.szFilePath, szFullPath, MAX_PATH - 1 );
	
	// ------------------------------------------
	// If the request is CGI,
	//     return special result 999 and force to do CGI function.
	// ------------------------------------------
	if( bIsCgi )
		return 999;	// CGI

	//-------------------------------------------
	// If-Modified-Since:
	// @RFC2616 sec.14.25 If-Modified-Since
	//-------------------------------------------
	char* pReq = m_hashReqHeader.GetValueI("if-modified-since");
	
	if( pReq != NULL )
	{
		if( MyTime( pReq ) >= MyTime((m_hrHttpReq.wfdFile.ftLastWriteTime)) )
		{
			return 304;
		}
	}

	return 200;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check basic authentication
//	@param
//		nHttpStatus		HTTP status
//		pszFilePath		Authentication file's path
//		pszBAuthFile	Authentication file
//	@return
//		nHttpSatus	Read success
//		401			Need authentication.
//		403			Authentication failed.
//		500			InternalServerError
//	http://www.ietf.org/rfc/rfc2617.txt
//  RFC 2617
//-----------------------------------------------------------------------------
// -- smaple --
// WWW-Authenticate: Basic realm="localhost"
// HTTP/1.1 401 Access Denied
//-----------------------------------------------------------------------------
int MyHttpSock::CheckBasicAuth(int nHttpStatus, const char *pszFilePath, const char *pszBAuthFile)
{
	char	szFile[MAX_PATH];	// tmp
	char*	pszSerchPath;
	char*	p;
	int		nLen;
	int		nDocRootLen;
	int		nAuthFileLen;
	HANDLE hFile;
	WIN32_FIND_DATA FindFileData;

	// Initialize
	memset( szFile, 0, MAX_PATH );
	nDocRootLen = strlen( m_pServerInfo->pszDocRootName );
	nAuthFileLen = strlen( pszBAuthFile );

	if( NULL == ( pszSerchPath = STRDUP(pszFilePath) ) )
	{
		return 500;
	}

	// access.cha?
	nLen = strlen( pszSerchPath );
	p = &pszSerchPath[nLen];
	while( *p != '\\' && p != pszSerchPath ) p--;
	p++;
	
	if( 0 == STRICMP( pszBAuthFile, p ) )
	{
		FREE( pszSerchPath );
		return 404;		// Auth file always can't be 'GET'.
	}

	do
	{
		// Back pointer to end of \.
		// \aaa\bbb\ccc.ddd -> \aaa\bbb
		nLen = strlen( pszSerchPath );
		p = &pszSerchPath[nLen];
		while( *p != '\\' && p != pszSerchPath ) p--;
		*p = '\0';

		if( (nLen = strlen( pszSerchPath )) < nDocRootLen )
			break;

		// Serch path arrived to root
		if( pszSerchPath[0] == '\0' )
			break;

		strcpy( szFile, pszSerchPath );
		strcat( szFile, "\\" );
		strcat( szFile, pszBAuthFile );

		hFile = FindFirstFileMBCS( szFile, &FindFileData );

		if( hFile != INVALID_HANDLE_VALUE )
		{
			FindClose( hFile );

			// Find but it was directory
			if( FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
			{
				continue;
			}
			
			FREE( pszSerchPath );

			return CheckBasicAuthFile(nHttpStatus, szFile);
		}

	}while( 1 );

	if( pszSerchPath != NULL )
	{
		FREE( pszSerchPath );
	}

	return nHttpStatus;
}

//-----------------------------------------------------------------------------
//	@detail
//		Read a authentication file to the memory.
//		work under CheckBasicAuth
//	@param
//		
//	@return
//		nHttpSatus	Read success
//		401			Need authentication.
//		403			Authentication failed.
//		500			InternalServerError
//	@comment
//		Sequence of retry is automaticaly processed by a blowser,
//		then I return 401 always.
//-----------------------------------------------------------------------------
// -- sample --
// GET / HTTP/1.0
// Authorization: Basic aXNoaTo=
// User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)
// Host: localhost
// Connection: close
//-----------------------------------------------------------------------------
int MyHttpSock::CheckBasicAuthFile(int nHttpStatus, char* pszAuthFilePath)
{
	// --------------------------------------------------------------
	// Request header check.
	// --------------------------------------------------------------
	char* p = m_hashReqHeader.GetValueI("authorization");
	char szUserName[64] = {'\0'};

	if( p == NULL )
		return 401; // Access Denied

	if( STRNICMP(p, "basic", 5) != 0 )
		return 401;

	p += 5;
	while( *p == ' ' ) p++;

	// base64 decording.
	p = strncpy(szUserName, p, 63);
	
	Base64Decord((unsigned char*)p);

	// --------------------------------------------------------------
	// Open password file.
	// --------------------------------------------------------------
	int	nFile = strlen(pszAuthFilePath);
	WCHAR*	pwszFile = new WCHAR[nFile+1];

	if( pwszFile == NULL )
	{
		// 500 InternalServerError
		return 500;
	}
	memset( pwszFile, 0, sizeof(WCHAR)*(nFile+1) );
	
	if( 0 == MultiByteToWideChar(CP_ACP,0,pszAuthFilePath,-1,pwszFile, nFile+1) )
	{
		delete[] pwszFile;
		return 500;	// 500 InternalServerError
	}

	// --------------------------------------------------------------
	HANDLE hFile = CreateFile(pwszFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	delete[] pwszFile;
	MyVarMem	vm;

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return 500;	// Internal Server Error
	}
	else
	{
		char	szBuf[256];
		BOOL	bResult;
		DWORD	dwReadBytes;
		do
		{
			bResult = ReadFile( hFile, szBuf, 256, &dwReadBytes, NULL );
			
			if(dwReadBytes == 0)
				break;

			if( 0 == vm.Add( szBuf, dwReadBytes ) )
			{
				CloseHandle(hFile);
				return 500;	// no memory
			}

		}while( bResult );

		CloseHandle(hFile);
	}

	// Do nothing if file length is 0.
	if( vm.GetLength() == 0 )
	{
		return nHttpStatus;
	}

	// --------------------------------------------------------------
	// user:password check.
	// --------------------------------------------------------------
	char*	pPasswd;
	char*	pNextPasswd;

	pPasswd = pNextPasswd = (char*)vm.GetBuffer();

	do
	{
		while( *pNextPasswd == '\r' ||  *pNextPasswd == '\n' )
			pNextPasswd++;

		if( *pNextPasswd == '\0' )
			return 401;//403;

		pPasswd = pNextPasswd;

		while( *pNextPasswd != '\r' &&  *pNextPasswd != '\n' && *pNextPasswd != '\0' )
		{
			pNextPasswd++;
		}
		if( *pNextPasswd != '\0' )
			*pNextPasswd++ = '\0';
		
		if( strcmp( pPasswd, p ) == 0 )
		{	
			// Authentication success.
			// make user name for access_log.
			char szUserName[16];
			for( int ii = 0; (ii < 15) && (*pPasswd != ':') && (*pPasswd != '\0'); ii++, pPasswd++ )
			{
				szUserName[ii] = *pPasswd;
			}
			szUserName[ii] = '\0';
			m_hashReqHeader.Set("authentication_user", szUserName);
			return nHttpStatus;
		}

		// next line
	}while( *pNextPasswd != '\0' );

	return 401;//403;

}

//-----------------------------------------------------------------------------
//	@detail
//		Make Environment variable.
//	@param
//	@return
//-----------------------------------------------------------------------------
void MyHttpSock::MakeEnvironment()
{
	char work[HEADER_LINE_MAX] = {0};
	
	// ------------------------------------------------------------------------
	// CONTENT_XXX
	// ------------------------------------------------------------------------
	sprintf(work, "%u", m_hReqBody.GetLength());
	m_hashEnv.Set("CONTENT_LENGTH",		work);
	m_hashEnv.Set("CONTENT_TYPE",		m_hashReqHeader.GetValueI("content-type"));
	
	// ------------------------------------------------------------------------
	// QUERY_XXX
	// ------------------------------------------------------------------------
	char* pQS = m_hashReqHeader.GetValue("QUERY_STRING");
	if( NULL == pQS ) pQS = "";
	m_hashEnv.Set("QUERY_STRING", pQS);
	
	// ------------------------------------------------------------------------
	// REMOTE_XXX
	// ------------------------------------------------------------------------
	sockaddr_in* psa = m_hrHttpReq.psaRemoteHost;
	u_char* pIP = (u_char*)&(psa->sin_addr.S_un.S_un_b);
	sprintf( work, "%u.%u.%u.%u", pIP[0], pIP[1], pIP[2], pIP[3] );
	m_hashEnv.Set("REMOTE_ADDR", work );
	
	char* pRH = m_hashReqHeader.GetValueI(LOG_REMOTE_HOST);
	if( pRH == NULL ) m_hashEnv.Set( "REMOTE_HOST", work );

	sprintf( work, "%d", psa->sin_port );
	m_hashEnv.Set( "REMOTE_PORT", work );


	// ------------------------------------------------------------------------
	// REQUEST_XXX
	// ------------------------------------------------------------------------
	switch(m_hrHttpReq.HttpCmd)
	{
		case HTTPHEAD:
			m_hashEnv.Set("REQUEST_METHOD",		"HEAD");
			break;
		case HTTPGET:
			m_hashEnv.Set("REQUEST_METHOD",		"GET");
			break;
		case HTTPPOST:
			m_hashEnv.Set("REQUEST_METHOD",		"POST");
			break;
		default:
			m_hashEnv.Set("REQUEST_METHOD",		"");
			break;
	}

	// ------------------------------------------------------------------------
	// SERVER_XXX
	// ------------------------------------------------------------------------
	m_hashEnv.Set("SERVER_NAME",		m_hashReqHeader.GetValueI("host"));
	sprintf(work, "%u",					m_pServerInfo->nPort);
	m_hashEnv.Set("SERVER_PORT",		work);
	m_hashEnv.Set("SERVER_PROTOCOL",	m_hrHttpReq.szHttpVer);
	m_hashEnv.Set("SERVER_SOFTWARE",	m_pServerInfo->pszSoftware);
	
	// ------------------------------------------------------------------------
	// DOCUMENT_XXX
	// ------------------------------------------------------------------------
	m_hashEnv.Set("DOCUMENT_ROOT",		m_pServerInfo->pszDocRootName);

	
	// ------------------------------------------------------------------------
	// HTTP_XXX
	// ------------------------------------------------------------------------
	m_hashReqHeader.SeekToFirst();

	while(m_hashReqHeader.SeekNext())	// to avoid RequestLine(e.g. GET / ... )
	{
		char* pwk = work;
		STRCPY( work, "HTTP_" );	// <- length is 5
		STRNCAT( work, m_hashReqHeader.GetCurrentKey(), HEADER_LINE_MAX-5-1 );
		STRUPR( work );
		for( pwk = work; *pwk != '\0'; pwk++ )
		{
			if( *pwk == '-' ) *pwk = '_';
		}
		m_hashEnv.Set( work,	m_hashReqHeader.GetCurrentValue() );

	};
	
	/*
	m_hashEnv.Set("HTTP_ACCEPT",		m_hashReqHeader.GetValueI("accept"));
	m_hashEnv.Set("HTTP_USER_AGENT",	m_hashReqHeader.GetValueI("user-agent"));
	m_hashEnv.Set("HTTP_COOKIE",		m_hashReqHeader.GetValueI("cookie"));
	m_hashEnv.Set("HTTP_REFERER",		m_hashReqHeader.GetValueI("referer"));
	*/	
}

//-----------------------------------------------------------------------------
//	@detail
//		Cleanup memory.
//	@param
//	@return
//-----------------------------------------------------------------------------
void MyHttpSock::FreeMemory()
{
	m_hReqHead.Clear();
	m_hReqBody.Clear();
	m_hashReqHeader.Clear();
	m_hashEnv.Clear();
}

//-----------------------------------------------------------------------------
//	@detail
//		Load specified chx file and execute.
//	@param
//		Full path of chx file.
//	@return
//-----------------------------------------------------------------------------
int MyHttpSock::ExecuteChx(char* pszFile, int bClose)
{
	// --------------------------
	// transform filename to unicode
	// --------------------------
	WCHAR	wszFile[MAX_PATH];

	if( 0 == MultiByteToWideChar(CP_ACP, 0, pszFile, -1, wszFile, MAX_PATH) )
	{
		return SendDefaultHttpResponse(500, bClose, __LINE__);
	}

	// --------------------------
	// execute
	// --------------------------
	char szHeadBuf[SOCKET_HEAD_BUF];

	m_hashEnv.Set("GATEWAY_INTERFACE",	"CHX/1.0");

	// --------------------------
	// Call CHX
	// --------------------------
	MyVarMem hRes;
	CEHTTP_CHX_EXTENTION pChx;

	HMODULE hDLL = ::LoadLibrary(wszFile);

	if( hDLL == NULL )
	{
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	pChx = (CEHTTP_CHX_EXTENTION)GetProcAddress( hDLL, FNC_EXTEND_CHX );

	if( !pChx )
	{
		FreeLibrary(hDLL);
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}
	
	m_hashEnv.SeekToFirst();

	int nCgiResponseLen = pChx( &m_hashEnv, &m_hReqBody, &hRes );

	// ------------------------------
	// Response has location header?
	// ------------------------------
	char* phRes = (char*)hRes.GetBuffer();

	if( phRes == NULL )
	{
		FreeLibrary(hDLL);
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	char* pRet = strstr( (const char*)phRes, "\r\n\r\n" );
	
	int nHeadlen;

	if( pRet != NULL )
	{
		nHeadlen = pRet - (const char*)hRes + 4;
	}
	else
	{
		FreeLibrary(hDLL);
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	// ------------------------------
	// If the body begin with 'Location:',
	// Relocation point header will be send.
	// notice : It is very simple check.
	// ------------------------------
	if( 0 == STRNICMP( hRes, "Location: ", 10 ) )
	{
		// ------------------------------
		// HTTP/1.x 302 Object Moved
		// Server: xxxxx
		// ------------------------------
		sprintf(szHeadBuf, 
			"%s %s\r\n"
			"Server: %s\r\n"
			"Connection: Close\r\n"

			, m_hrHttpReq.szHttpVer, hsStatus[HTTPSTATUS_302].pszString
			, m_pServerInfo->pszSoftware);
	}
	else
	{
		// ------------------------------
		// HTTP/1.x 200 OK
		// Server: xxxxx
		// ------------------------------
		sprintf(szHeadBuf, 
			"%s %s\r\n"
			"Server: %s\r\n"
			"Connection: Close\r\n"
			"Content-Length: %u\r\n"

			, m_hrHttpReq.szHttpVer, hsStatus[0].pszString
			, m_pServerInfo->pszSoftware
			, hRes.GetLength() - nHeadlen
			);
	}

	// Send default header.
	if( (SendData(szHeadBuf) < 1))
	{
		FreeLibrary(hDLL);
		return 0;
	}

	// Remember the length for log.
	sprintf( szHeadBuf, "%u", hRes.GetLength() - nHeadlen );
	m_hashReqHeader.Set( LOG_SEND_DATA, szHeadBuf );


	// ------------------------------
	// Send chx data
	// Content-Type: XXXX/XXXX
	// 
	// <data...>
	// ------------------------------
	if( m_hrHttpReq.HttpCmd ==  HTTPHEAD )
	{
		if( SendData( hRes, nHeadlen ) < 1) return 0;
	}
	else
	{
		if( SendData( hRes, hRes.GetLength() + 1 ) < 1) return 0;
	}

	FreeLibrary(hDLL);
	return 999;

}

//-----------------------------------------------------------------------------
//	@detail
//		Load specified dll file and execute.
//	@param
//		Full path of dll file.
//	@return
//-----------------------------------------------------------------------------
int MyHttpSock::ExecuteDll(char *pszFile, int bClose)
{
	// --------------------------
	// SCRIPT_NAME
	// --------------------------
	int nDRootLen = strlen(m_pServerInfo->pszDocRootName);
	m_hashEnv.Set("SCRIPT_NAME", &pszFile[nDRootLen]);
	bslash2slash(m_hashEnv.GetValue("SCRIPT_NAME"));
	m_hashEnv.Set("GATEWAY_INTERFACE",	"CHX/1.0");

	// --------------------------
	// transform filename to unicode
	// --------------------------
	WCHAR	wszFile[MAX_PATH];

	if( 0 == MultiByteToWideChar(CP_ACP, 0, pszFile, -1, wszFile, MAX_PATH) )
	{
		return SendDefaultHttpResponse(500, bClose, __LINE__);
	}

	// --------------------------
	// Call DLL
	// --------------------------
	CEHTTP_DLL_EXTENTION pCAPI;

	HMODULE hDLL = ::LoadLibrary(wszFile);
	if( hDLL == NULL )
	{
		return SendDefaultHttpResponse(500, bClose, __LINE__);
	}

	pCAPI = (CEHTTP_DLL_EXTENTION)GetProcAddress(hDLL, FNC_EXTEND_DLL);

	if( !pCAPI )
	{
		FreeLibrary(hDLL);
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	void* pCgiResponse = NULL;

	int nCgiResponseLen = pCAPI(
		m_hashEnv.GetBuffer(), m_hashEnv.GetLength(),
		m_hReqHead.GetBuffer(), m_hReqHead.GetLength(),
		m_hReqBody.GetBuffer(), m_hReqBody.GetLength(),
		&pCgiResponse);

	if( NULL != pCgiResponse )
	{
		if( SendData((const char*)pCgiResponse, nCgiResponseLen) < 1)
		{
			FreeLibrary(hDLL);
			FREEIF(pCgiResponse);
			return 0;
		}
	}
	FreeLibrary(hDLL);
	
	return 999;
}

//-----------------------------------------------------------------------------
//	@detail
//		Load specified exe file and execute.
//	@param
//		Full path of exe file.
//	@return
//
//	@Refferer
//		KB190351:http://support.microsoft.com/default.aspx?scid=kb;EN-US;190351
//		KB150956:http://support.microsoft.com/default.aspx?scid=kb;EN-US;150956
//-----------------------------------------------------------------------------
int MyHttpSock::ExecuteCGI(char *pszFile, int bClose)
{
#if defined(_WIN32_WCE)
	
	return SendDefaultHttpResponse( 403, bClose, __LINE__ );

#elif defined(_WIN32)
	m_hashEnv.Set("GATEWAY_INTERFACE",	"CGI/1.1");
	
	HANDLE	hPrcs = GetCurrentProcess();

	HANDLE	hstdIn[2] = {NULL};
	HANDLE	hstdOut[2] = {NULL};
	HANDLE	hstdErr[2] = {NULL};
	HANDLE	hstdInDup = NULL;
	HANDLE	hstdOutDup = NULL;
	HANDLE	hstdErrDup = NULL;

	SECURITY_ATTRIBUTES sa = {0};

	sa.nLength = sizeof( SECURITY_ATTRIBUTES );
	sa.bInheritHandle = TRUE;
	sa.lpSecurityDescriptor = NULL;

	// ----------------------------------------------------
	// STDIN WebServer -> CGI
	if( 0 == CreatePipe( &hstdIn[RPIPE], &hstdIn[WPIPE], &sa, 0 ) )
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );


	// ----------------------------------------------------
	// STDOUT WebServer <- CGI
	if( 0 == CreatePipe( &hstdOut[RPIPE], &hstdOut[WPIPE], &sa, 0 ) )
	{
		CloseHandle( hstdIn[RPIPE] ); CloseHandle( hstdIn[WPIPE] );
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	// ----------------------------------------------------
	// STDERR WebServer <- CGI
	if( 0 == CreatePipe( &hstdErr[RPIPE], &hstdErr[WPIPE], &sa, 0 ) )
	{
		CloseHandle( hstdIn[RPIPE] ); CloseHandle( hstdIn[WPIPE] );
		CloseHandle( hstdOut[RPIPE] ); CloseHandle( hstdOut[WPIPE] );
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}
	
	// ----------------------------------------------------
	// DuplicateHandle
	if(	!DuplicateHandle( hPrcs, hstdIn[WPIPE], hPrcs, &hstdInDup, 0, FALSE, DUPLICATE_SAME_ACCESS )
		|| !DuplicateHandle( hPrcs, hstdOut[RPIPE], hPrcs, &hstdOutDup, 0, FALSE, DUPLICATE_SAME_ACCESS )
		|| !DuplicateHandle( hPrcs, hstdErr[RPIPE], hPrcs, &hstdErrDup, 0, FALSE, DUPLICATE_SAME_ACCESS ) )
	{
		CloseHandle( hstdIn[RPIPE] ); CloseHandle( hstdIn[WPIPE] );
		CloseHandle( hstdOut[RPIPE] ); CloseHandle( hstdOut[WPIPE] );
		CloseHandle( hstdErr[RPIPE] ); CloseHandle( hstdErr[WPIPE] );
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	CloseHandle( hstdIn[WPIPE] );
	hstdIn[WPIPE] = hstdInDup;

	CloseHandle( hstdOut[RPIPE] );
	hstdOut[RPIPE] = hstdOutDup;
	
	CloseHandle( hstdErr[RPIPE] );
	hstdErr[RPIPE] = hstdErrDup;
	
	// ----------------------------------------------------
	// Create STARTUPINFO
	STARTUPINFO si = {0};

	si.cb = sizeof( STARTUPINFO );
	si.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;
	si.hStdInput = hstdIn[RPIPE];
	si.hStdOutput = hstdOut[WPIPE];
	si.hStdError = hstdErr[WPIPE];

	PROCESS_INFORMATION pi = {0};

	// ----------------------------------------------------
	// Make environment
	MyVarMem vmEnv;
	m_hashEnv.SeekToFirst();


	char* pKey = m_hashEnv.GetCurrentKey();

	if( pKey != NULL )
	{

		do{
			char* pVal = m_hashEnv.GetCurrentValue();
	
			vmEnv.Add( pKey );
			vmEnv.Add( "=" );
			if( *pVal != '\0' )
			{
				vmEnv.Add( pVal );
			}
			vmEnv.Add( "", 1 );
	
		}while( pKey = m_hashEnv.SeekNext() );
	}

	// ----------------------------------------------------
	// Make execute infomations
	WCHAR* pwstrExe = malloc_char2wchar( pszFile );
	WCHAR* pwstrCurDir = malloc_char2wchar( pszFile );
	WCHAR* p = pwstrCurDir;	// Make CurrentDir
	while( *p != '\0' ) p++;
	while( *p != '\\' ) *p-- = '\0';

	// ----------------------------------------------------
	// Spawn cgi process
	BOOL bProcess = CreateProcess( 
		pwstrExe, NULL, NULL, NULL, TRUE, DETACHED_PROCESS, vmEnv.GetBuffer(), pwstrCurDir, &si, &pi );

	free(pwstrExe);
	free(pwstrCurDir);

	if( !bProcess )
	{
		// Close all handles
		CloseHandle( hstdIn[RPIPE] );  CloseHandle( hstdIn[WPIPE] );
		CloseHandle( hstdOut[RPIPE] ); CloseHandle( hstdOut[WPIPE] );
		CloseHandle( hstdErr[RPIPE] ); CloseHandle( hstdErr[WPIPE] );
		return SendDefaultHttpResponse( 500, bClose, __LINE__ );
	}

	CloseHandle( hstdIn[RPIPE] );
	CloseHandle( hstdOut[WPIPE] );
	CloseHandle( hstdErr[WPIPE] );

	CloseHandle( pi.hThread );

	DWORD dwWritten = 0;
	DWORD dwRead = 0;

	// ----------------------------------------------------
	// send post data
	if( m_hrHttpReq.HttpCmd == HTTPPOST )
	{
		WriteFile( hstdIn[WPIPE], m_hReqBody.GetBuffer(), m_hReqBody.GetLength(), &dwWritten, NULL );
	}

	// Close unused pipe
	CloseHandle( hstdIn[WPIPE] );

	char	szBuf[1024] = {0};
	DWORD	dwTotalLength = 0;
	char	szHeaderLine[64] = {0};
	int		fSentHeader = 0;
	MyVarMem	hRes;


	// ----------------------------------------------------
	// 1 pass : Header serch.
	while(TRUE)
	{
		if (!ReadFile( hstdOut[RPIPE], szBuf, sizeof(szBuf), &dwRead, NULL ) || !dwRead )
		{
//			if (GetLastError() == ERROR_BROKEN_PIPE)
//			{
//				break; // pipe done - normal exit path.
//			}
//			else
//			{
				CloseHandle( hstdOut[RPIPE] );
				CloseHandle( hstdErr[RPIPE] );
				CloseHandle( pi.hProcess );

				return SendDefaultHttpResponse( 500, bClose, __LINE__ );
//			}
		}

		if( 0 == hRes.Add( szBuf, dwRead ) )
		{
			CloseHandle( hstdOut[RPIPE] );
			CloseHandle( hstdErr[RPIPE] );
			CloseHandle( pi.hProcess );

			return SendDefaultHttpResponse( 500, bClose, __LINE__ );
		}

		if( hRes.GetLength() > 10 )	//10 is strlen( "Location: " )
		{
			char szHeadBuf[SOCKET_HEAD_BUF];

			if( 0 == STRNICMP( hRes, "Location: ", 10 ) )
			{
				// ------------------------------
				// HTTP/1.x 302 Object Moved
				// Server: xxxxx
				// ------------------------------
				sprintf(szHeadBuf, 
					"%s %s\r\n"
					"Server: %s\r\n"
					"Connection: Close\r\n"

					, m_hrHttpReq.szHttpVer, hsStatus[HTTPSTATUS_302].pszString
					, m_pServerInfo->pszSoftware);
			}
			else
			{
				// ------------------------------
				// HTTP/1.x 200 OK
				// Server: xxxxx
				// ------------------------------
				sprintf(szHeadBuf, 
					"%s %s\r\n"
					"Server: %s\r\n"
					"Connection: Close\r\n"

					, m_hrHttpReq.szHttpVer, hsStatus[0].pszString
					, m_pServerInfo->pszSoftware
					);
			}

			if( SendData( szHeadBuf ) < 0 )
			{
				CloseHandle( hstdOut[RPIPE] );
				CloseHandle( hstdErr[RPIPE] );
				CloseHandle( pi.hProcess );
				return 0;
			}

			if( SendData( hRes, hRes.GetLength() ) < 0 )
			{
				CloseHandle( hstdOut[RPIPE] );
				CloseHandle( hstdErr[RPIPE] );
				CloseHandle( pi.hProcess );
				return 0;
			}

			// we done send headers and some body data, break.
			break;
		}
		
	}

	// ----------------------------------------------------
	// 2 pass : Burst mode
	while(TRUE)
	{
		if (!ReadFile( hstdOut[RPIPE], szBuf, sizeof(szBuf), &dwRead, NULL ) || !dwRead )
		{
//			if (GetLastError() == ERROR_BROKEN_PIPE)
//			{
//				break; // pipe done - normal exit path.
//			}
//			else
//			{
//				break;
//			}
			break;
		}

		if( SendData( szBuf, dwRead ) < 0 )
		{
			CloseHandle( hstdOut[RPIPE] );
			CloseHandle( hstdErr[RPIPE] );
			CloseHandle( pi.hProcess );
			return 0;
		}
	}

	CloseHandle( hstdOut[RPIPE] );
	CloseHandle( hstdErr[RPIPE] );
	CloseHandle( pi.hProcess );

	return 200;

#else
	return 403;
#endif
}

//-----------------------------------------------------------------------------
//	@detail
//		Devide the request header by ':'.
//		The head of space character is ignored.
//	@param
//		pReq		Request header
//		szReqKey	Key pointer
//		szReqValue	Value pointer
//	@return
//		The number of sentence.
//-----------------------------------------------------------------------------
static int ParseString( const char* pReq, char* szReqKey, char* szReqValue )
{
	if( pReq == NULL || *pReq == '\0' )
		return 0;
	*szReqKey = *pReq;

	while( *pReq != ':' )
	{
		if( *pReq == ' ' || *pReq == '\t' )
		{
			*pReq++;
			continue;
		}
		else if( *pReq == '\r' || *pReq == '\n' || *pReq == '\0' )
		{
			*szReqKey = '\0';
			return 1;
		}
		*szReqKey = *pReq;
		szReqKey++;
		pReq++;
	}
	pReq++;
	*szReqKey = '\0';

	while( *pReq == ' ' || *pReq == '\t' ) pReq++;

	while( *pReq != '\0' && *pReq != '\r'  && *pReq != '\n')
	{
		*szReqValue++ = *pReq++;
	}
	*szReqValue = '\0';
	return 2;
}

//-----------------------------------------------------------------------------
//	@detail
//		Parse the request line.
//
//			format : Method SP Request-URI SP HTTP-Version CRLF
//
//	@param
//		pReq		Request data from browser.
//						It must be assuranceed that contain '\r\n'.
//		szCmd		[out]HEAD,GET,POST, ...
//						NOTICE: This function is not care the size of szCmd.
//		szPath		[out]Request file
//		szHttpVer	[out]HTTP version
//	@return
//		The number of the acquired data.
//-----------------------------------------------------------------------------
static int ParseRequestLine( const char* pReq, char* szCmd, char* szPath, char* szHttpVer )
{
	
	if( pReq == NULL || *pReq == '\0' )
		return 0;

	// szCmd
	while( *pReq != ' ' && *pReq != '\t' )
	{
		if( *pReq == '\0' || *pReq == '\r' || *pReq == '\n' )
		{
			*szCmd = '\0';
			return 1;
		}
		*szCmd++ = *pReq++;
	}
	*szCmd = '\0';
	pReq++;

	// szPath
	while( *pReq != ' ' && *pReq != '\t' )
	{
		if( *pReq == '\0' || *pReq == '\r' || *pReq == '\n' )
		{
			// When HTTP version is not specified, consider as HTTP/1.0
			*szPath = '\0';
			strcpy(szHttpVer, "HTTP/1.0");
			return 3;
		}
		*szPath++ = *pReq++;
	}
	*szPath = '\0';
	pReq++;

	// szHttpVer
	while( *pReq != '\0' && *pReq != '\r' && *pReq != '\n' )
	{
		*szHttpVer++ = *pReq++;
	}
	*szHttpVer = '\0';
	return 3;

}

//-----------------------------------------------------------------------------
//	@detail
//				Transelate absolute path to reativity path.
//
//					http://<server>:<port>/<reativity path>...
//	@param
//		char*	pszSrc			absolute path (or reativity path also ok)
//	@return
//		not safty	: 0
//		safty		: 1
//	@out
//-----------------------------------------------------------------------------
static int AbsoluteURL2relativityURL(char* pszAbsolutePath)
{
	if( pszAbsolutePath == NULL )
		return 0;

	char *pSrc = pszAbsolutePath;
	char *pDst = pszAbsolutePath;

	if( 0 == STRNICMP( "http://",  pSrc, 7 ) )
	{
		pSrc += 7;
		while( *pSrc != '/' )
		{
			if( *pSrc == '\0' )
			{
				// In case which having no '/', it may be virus request.
				return 0;
			}
			pSrc++;
		}

		while( *pSrc != '\0' )
		{
			*pDst++ = *pSrc++;
		}

		*pDst++ = *pSrc++;
	}
	return 1;

}

//-----------------------------------------------------------------------------
//	@detail
//				Check the request path.
//	@param
//		char*	pszSrc			reativity path
//	@return
//		not safty	: 0
//		safty		: 1
//	@out
//-----------------------------------------------------------------------------
static int DirectoryAccessCheck(const char* pszRelativePath)
{
	int count = 0;
	
	if( pszRelativePath == NULL )
		return 0;
	
	char* p = (char*)pszRelativePath;

	do{
		// \x	+1
		// \.	 0
		// ..	-1

		if( ( p[0] == '\\' || p[0] == '/' ) && p[1] != '.' )
			count++;
		else if( p[0] == '.' && p[1] == '.' )
			count--;

		if( count < 1 )
			return 0;

	}while( *++p != '\0' );

	return 1;
}
