#include "stdafx.h"
#include <stdlib.h>
#include "MySock.h"
#include "MyVarMem.h"

#if defined(_WIN32) && !defined(_WIN32_WCE)
#pragma comment( lib , "ws2_32.lib" )
#elif defined(_WIN32_WCE)
#pragma comment( lib , "winsock.lib" )
#endif

MySock::MySock()
{
	// init winsock
	InitDll();
	m_status = 0;
	m_socket = INVALID_SOCKET;
	// UDP/IP was not supported.
	m_nProtocol = SOCK_STREAM;

	// default time out.
	SetTimeOut(5,0);
}

MySock::MySock(SOCKET s)
{
	// init winsock
	InitDll();
	m_status = 0;
	m_socket = s;
	// UDP/IP was not supported.
	m_nProtocol = SOCK_STREAM;

	// default time out.
	SetTimeOut(5,0);

	if(m_socket == INVALID_SOCKET)
	{
		return;
	}

	int ioctl_opt =1;
	
	if (ioctlsocket(m_socket, FIONBIO,(unsigned long*)&ioctl_opt) == SOCKET_ERROR)
	{
		return;
	}

}

MySock::~MySock()
{
	Close();
}

//-----------------------------------------------------------------------------
//	@detail
//		start accepting
//	@param
//	@return
//		success		:SOCKET discripter
//		fail		:SOCKET_ERROR
//		timeout		:0
//-----------------------------------------------------------------------------
SOCKET MySock::Accept()
{
	sockaddr_in from;
	return Accept(&from);
}

SOCKET MySock::Accept(sockaddr_in* pFrom)
{
	fd_set	readfds;
	int i = 0;

	FD_ZERO(&readfds);
	FD_SET(m_socket,&readfds);
	

	i = ::select(0, &readfds, NULL, NULL, &m_timeout);

	// Socket error		i == SOCKET_ERROR
	// Timeout			i == 0
	if( i < 1 )
	{
		return INVALID_SOCKET;
	}

	int fromlen = sizeof(sockaddr_in);
	return ::accept(m_socket, (sockaddr*)pFrom, &fromlen);
}

//-----------------------------------------------------------------------------
//	@detail
//		Start listen
//	@param
//		int nPort	portnumber
//	@return
//		success		:SOCKET
//		fail		:SOCKET_ERROR
//-----------------------------------------------------------------------------
int MySock::Listen(int nPort)
{
	if(m_socket != INVALID_SOCKET)
	{
		Close();
	}

	sockaddr_in local;

	local.sin_family = AF_INET;			// TCP/IP
	local.sin_addr.s_addr = INADDR_ANY;	// all of NIC
	local.sin_port = htons(nPort);		// port number

	m_socket = socket(AF_INET, SOCK_STREAM,0);

	if (m_socket == INVALID_SOCKET)
	{
		return SOCKET_ERROR;
	}
	//	bind
	if (::bind(m_socket,(struct sockaddr*)&local,sizeof(local) ) == SOCKET_ERROR)
	{
		return SOCKET_ERROR;
	}
	// listen
	if (::listen(m_socket,SOMAXCONN) == SOCKET_ERROR)
	{
		return SOCKET_ERROR;
	}

	// non blocking option
	int ioctl_opt = 1;
	if (::ioctlsocket(m_socket,FIONBIO,(unsigned long*)&ioctl_opt) == SOCKET_ERROR)
	{
		return SOCKET_ERROR;
	}

	return 1;

}

//-----------------------------------------------------------------------------
//	@detail
//		Init Winsock
//	@param
//	@return		: TRUE
//-----------------------------------------------------------------------------
int MySock::InitDll()
{
	static initdone = 0;
    if (!initdone)
	{
		initdone = 1;

		WSADATA	wsaData;
		int winsockstartup = WSAStartup(WINSOCK_VERSION_REQUIRED,&wsaData);
		
		if (winsockstartup != 0 ||
			wsaData.iMaxSockets < WINSOCK_MIN_SOCKETS_REQUIRED)
		{
			WSACleanup();
			return FALSE;
		}
    }
	return TRUE;
}

//-----------------------------------------------------------------------------
//	@detail
//		Connect
//	@param
//		char*	ipaddr	:IP address or host name
//		char*	port	:Port number
//	@return
//		success			:0
//		fail			:SOCKET_ERROR
//-----------------------------------------------------------------------------
int MySock::Connect(const char* ipaddr, const char* port)
{
	DWORD ip;

	ip = inet_addr(ipaddr);

	if (ip == INADDR_NONE)
	{
		struct hostent* pHost;
		pHost = gethostbyname(ipaddr);
		
		if (pHost == NULL)
		{
			return SOCKET_ERROR;
		}
		
		ip = *((DWORD*)(pHost->h_addr));
	}

	SOCKADDR_IN sockaddr;
	sockaddr.sin_family = AF_INET;
	sockaddr.sin_port   = htons(atoi(port));
	sockaddr.sin_addr.s_addr = ip;
	m_socket = socket(AF_INET, SOCK_STREAM,0);

	m_status = ::connect(m_socket, (const struct sockaddr*)&sockaddr, sizeof(sockaddr));

	if (m_status != 0)
	{
		Close();
	}
	
	return m_status;
}

//-----------------------------------------------------------------------------
//	@detail
//		Send data
//	@param
//		char*	buf	: pointer to send buffer
//		int		len	: size of send data
//		int		flag:
//	@return
//		success		: data size of the sent data
//		fail		: SOCKET_ERROR
//		timeout		: 0
//-----------------------------------------------------------------------------
int MySock::SendData(const char *buf, int len, int flag)
{
	fd_set writefds;
	FD_ZERO(&writefds);
	FD_SET(m_socket, &writefds);
	int i = 0;

	if(m_socket == INVALID_SOCKET)
	{
		return SOCKET_ERROR;
	}
	
	if( len == 0 && buf != NULL)
		len = strlen(buf);

	if( len == 0 )
		return 0;

	int nTotalSend = 0;
	int nSendLen = 0;

	do
	{
		nSendLen = 0;
		i = ::select(0, NULL, &writefds, NULL, &m_timeout);

		// socket error		i == SOCKET_ERROR
		// timeout			i == 0
		if( i < 1 )
		{
			return SOCKET_ERROR;
		}

		// try to send data until time is out
		if( FD_ISSET(m_socket, &writefds) )
		{
			// nSendLen = ::send( m_socket, buf, len, flag );
			nSendLen = ::send( m_socket, &buf[nTotalSend], len - nTotalSend, flag );
			if( nSendLen > 0)
				nTotalSend += nSendLen;
			else
				break;
			
		}
	}while( nTotalSend < len );


	return nTotalSend;

}

//-----------------------------------------------------------------------------
//	@detail
//		Recive data from socket
//	@param
//		char*	buf	: pointer to the buffer
//		int		len	: size of the buffer
//		int		flag: 
//	@return
//		success		: size of recieved data
//		fail		: SOCKET_ERROR
//		timeout		: 0
//-----------------------------------------------------------------------------
int MySock::RecvData(char *buf, int len, int flag)
{
	fd_set	readfds;
	FD_ZERO(&readfds);
	FD_SET(m_socket,&readfds);
	int i = 0;
	int nRecvLen = 0;

	if(m_socket == INVALID_SOCKET)
	{
		return SOCKET_ERROR;
	}

	i = ::select(0, &readfds, NULL, NULL, &m_timeout);

	// socket error		i == SOCKET_ERROR
	// timeout			i == 0
	if( i < 1 )
	{
		return i;
	}

	// read
	if (FD_ISSET(m_socket, &readfds))
	{

		nRecvLen = ::recv(m_socket, buf, len, flag);
	}

	// MSDN says...
	// Zero indicates that the connection has been gracefully closed.
	// SOCKET_ERROR indicates failure.
	// To get a specific error value, call WSAGetLastError.
	if( nRecvLen == 0 )
	{
		return SOCKET_ERROR;
	}
	return nRecvLen;

}

//-----------------------------------------------------------------------------
//	@detail
//		close socket
//	@param
//	@return
//-----------------------------------------------------------------------------
void MySock::Close()
{
	if(m_socket != INVALID_SOCKET)
	{
		::shutdown(m_socket, 2);
		::closesocket(m_socket);
		m_socket = INVALID_SOCKET;
	}

}

//-----------------------------------------------------------------------------
//	@detail
//		Serch if the socket can read or can accept
//	@param
//	@return
//		see ::select
//		fail				:SOCKET_ERROR
//		timeout				:0
//		disconnected		:0
//-----------------------------------------------------------------------------
int MySock::IsReadalbe()
{
	fd_set	readfds;

	FD_ZERO(&readfds);
	FD_SET(m_socket,&readfds);

	return ::select(0, &readfds, NULL, NULL, &m_timeout);
}
