#include <windows.h>
#include <stdio.h>
#include "MyTime.h"

char* MyTime::m_pszMonth[12] =
{
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

char* MyTime::m_pszWkday[7] =
{
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

char* MyTime::m_pszWeekday[7] = 
{
	"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
};

// ----------------------------------------------------------------------------
MyTime::MyTime()
{
	GetSystemTime(&m_st);
	SystemTimeToFileTime(&m_st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const SYSTEMTIME& pSysTime)
{
	SystemTimeToFileTime(&pSysTime, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const FILETIME& FileTime)
{
	// Filetime is as UTC
	// convert that time to system time
	// FileTimeToSystemTime(&FileTime, &m_st);
	//m_ft.dwHighDateTime = pFileTime.dwHighDateTime;
	//m_ft.dwLowDateTime = pFileTime.dwLowDateTime;
	m_ft = FileTime;

	FileTimeToSystemTime(&FileTime, &m_st);
	m_st.wMilliseconds = 0;
	SystemTimeToFileTime(&m_st, &m_ft);

}

// ----------------------------------------------------------------------------
MyTime::~MyTime()
{
	memset( &m_st, 0, sizeof(SYSTEMTIME) );
}

// ----------------------------------------------------------------------------
void MyTime::SetRfc1123Time(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
	char szTime[32];
	int ii;

	strncpy(szTime, pszMyTime, 32);

	szTime[07] = '\0';
	szTime[11] = '\0';
	szTime[16] = '\0';
	szTime[19] = '\0';
	szTime[22] = '\0';
	szTime[25] = '\0';

	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[8]) == 0 )
		{
			ii++;
			break;
		}
	}
	if( ii > 12 )
		ii = 11;	// Bad Request

	m_st.wDay    = atoi(&szTime[5]);
	m_st.wMonth  = ii;
	m_st.wYear   = atoi(&szTime[12]);
	m_st.wHour   = atoi(&szTime[17]);
	m_st.wMinute = atoi(&szTime[20]);
	m_st.wSecond = atoi(&szTime[23]);
	m_st.wMilliseconds = 0;
	m_st.wDayOfWeek = DayOfWeek( m_st.wYear, m_st.wMonth, m_st.wDay );

	SystemTimeToFileTime(&m_st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::SetRfc1036Time(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
	char szTime[32];
	int ii;

	strncpy(szTime, pszMyTime, 32);

	szTime[10] = '\0';
	szTime[14] = '\0';
	szTime[17] = '\0';
	szTime[20] = '\0';
	szTime[23] = '\0';
	szTime[26] = '\0';


	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[11]) == 0 )
		{
			ii++;
			break;
		}
	}
	if( ii > 11 )
		ii = 11;	// Bad Request

	m_st.wDay    = atoi(&szTime[8]);
	m_st.wMonth  = ii;
	m_st.wYear   = atoi(&szTime[15]);
	m_st.wYear += ( m_st.wYear > 80 ) ? 1900:2000;
	m_st.wHour   = atoi(&szTime[18]);
	m_st.wMinute = atoi(&szTime[21]);
	m_st.wSecond = atoi(&szTime[24]);
	m_st.wMilliseconds = 0;
	m_st.wDayOfWeek = DayOfWeek( m_st.wYear, m_st.wMonth, m_st.wDay );

	SystemTimeToFileTime(&m_st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::SetAsctime(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
//	asctime-date = wkday SP date3 SP time SP 4DIGIT
//	date3        = month SP ( 2DIGIT | ( SP 1DIGIT ))
//                ; month day (e.g., Jun  2)

	char szTime[32];
	int ii;

	strncpy(szTime, pszMyTime, 32);

	szTime[07] = '\0';
	szTime[10] = '\0';
	szTime[13] = '\0';
	szTime[16] = '\0';
	szTime[19] = '\0';
	szTime[24] = '\0';

	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[11]) == 0 )
		{
			ii++;
			break;
		}
	}

	if( ii > 11 )
		ii = 11;	// Bad Request

	m_st.wDay    = atoi(&szTime[8]);
	m_st.wMonth  = ii;
	m_st.wYear   = atoi(&szTime[20]);
	m_st.wHour   = atoi(&szTime[11]);
	m_st.wMinute = atoi(&szTime[14]);
	m_st.wSecond = atoi(&szTime[17]);
	m_st.wMilliseconds = 0;
	m_st.wDayOfWeek = DayOfWeek( m_st.wYear, m_st.wMonth, m_st.wDay );

	SystemTimeToFileTime(&m_st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
//	Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
//	Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format

	if( pszMyTime[3] == ',' )
		SetRfc1123Time( pszMyTime );
	else if( pszMyTime[3] == ' ' )
		SetAsctime( pszMyTime );
	else
		SetRfc1036Time( pszMyTime );

}

// ----------------------------------------------------------------------------
const char*	MyTime::GetFormatRfc1123()
{
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
	FileTimeToSystemTime(&m_ft, &m_st);

	sprintf(m_szBuffer, "%s, %02u %s %04u %02u:%02u:%02u GMT",
		m_pszWkday[m_st.wDayOfWeek],
		m_st.wDay,
		m_pszMonth[m_st.wMonth - 1],
		m_st.wYear,
		m_st.wHour, m_st.wMinute, m_st.wSecond);

	return m_szBuffer;
}

// ----------------------------------------------------------------------------
SYSTEMTIME	MyTime::GetAsSystemTime() const
{
	SYSTEMTIME st;
	FileTimeToSystemTime(&m_ft, &st);
	return st;
}

// ----------------------------------------------------------------------------
int MyTime::DayOfWeek(int y, int m, int d)
{
	if (m < 3)
	{
		y--;
		m += 12;
	}
	return (y + y/4 - y/100 + y/400 + (26*m+16)/10 + d) % 7;
}

