/* =========================================================================
	This library is part of IshiSoft CEHTTP base class.
	------------------------------------------------------------------------
	author: yu-ishi@ya2.so-net.ne.jp
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#if !defined(_MYVARMEM_H_INCLUDED_)
#define _MYVARMEM_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

#if defined(WIN32)
	#include "windows.h"
	#define MALLOC( size )		HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, size )
	#define REALLOC( p, size )	HeapReAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, p, size )
	#define FREE( p )			HeapFree( GetProcessHeap(), HEAP_NO_SERIALIZE, p )
	#define FREEIF( p )			{if(p) HeapFree( GetProcessHeap(), HEAP_NO_SERIALIZE, p );p=NULL;}
#else
	#include <malloc.h>
	#define MALLOC( size )		malloc( size )
	#define REALLOC( p, size )	realloc( p, size )
	#define FREE( p )			free( p )
	#define FREEIF( p )			{if(p) free(p);p=NULL;}
#endif


/* ****************************************************************************

 variable memory class

	Expand its allocated memory on the demand.
	The paging size is 512Bytes

	basic structure of a memory

	data		:|<...  data  ...>|\0|	<- add the null terminate automatically.
	data length	:|<-m_nBufferLen->|		<- MyVarMem.GetLength() returns this score.
	real length	:|<-          512*n            ->|


**************************************************************************** */

class MyVarMem 
{
public:
	MyVarMem();
	virtual ~MyVarMem();

private:
	void*	m_pBuffer;		// Pointer to allocated buffer
	size_t	m_nBufferLen;	// Length of the contents
	int		m_nBlockSize;	// Block size

public:
	
	void* GetBuffer()	const	{return m_pBuffer;}
	size_t GetLength()	const	{return m_nBufferLen;}

	size_t AddHead( const void* pAddData, size_t nDataLen);
	size_t Add( const void* pAddData, size_t nDataLen);
	
	size_t AddHead(MyVarMem* pH)	{return AddHead(pH->GetBuffer(), pH->GetLength());}
	size_t Add(MyVarMem* pH)		{return Add(pH->GetBuffer(), pH->GetLength());}
	
	size_t Add(const char* pszStr);

	virtual void Clear();
	virtual int IsEmpty()	{return (m_pBuffer==NULL)?1:0;}

	operator const void*()	{return (const void*)m_pBuffer;}
	operator const char*()	{return (const char*)m_pBuffer;}
};
#endif // !defined(_MYVARMEM_H_INCLUDED_)
