/* =========================================================================
	This extention is part of IshiSoft CEHTTP.
	------------------------------------------------------------------------
	author: yu-ishi@ya2.so-net.ne.jp
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include <stdio.h>	// for PCHTTP
#include <stdlib.h>	// for PCHTTP
#include "MyHash.h"

CRITICAL_SECTION* pcs = NULL;

#define MAX_RECORD_LEN	512
const char* g_pNullString = "-";
const char* g_pszMonth[12] =
{
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static const char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey );
static char *STRNCPY( char *dst, const char *src, size_t count );
static unsigned int STRLEN( const char *string );

//-----------------------------------------------------------------------------
//	@detail
//		Create logfile extention
//		work on CEHTTP directory.
//	@param
//		WCHAR* pwszLogFile		// File to create
//		int nLogSize			// Limit of logfile
//		int nHttpStatus			// http status
//		MyHash* pHashHeaders	// headers
//	@return
//	@comment
//-----------------------------------------------------------------------------
int fnCehttpLog( WCHAR* pwszLogFile, int nLogSize, int nHttpStatus, MyHash* pHashHeaders )
{
	HANDLE hFile = INVALID_HANDLE_VALUE;
	DWORD dwWritten;
	char szBuf[MAX_RECORD_LEN] = {0};
	int	nRecordLen = 0;

	// ------------------------------------------------------------------------
	SYSTEMTIME st;
	GetLocalTime(&st);

	const char* pRemoteHost = pHashHeaders->GetValueI( "REMOTE_HOST" );
	if( NULL == pRemoteHost )
	{
		pRemoteHost	= GetValueAvoidNull( pHashHeaders, "remote_addr" );
	}
	const char* pRequestLine	= GetValueAvoidNull( pHashHeaders, "request_line" );
	const char* pSendBytes		= GetValueAvoidNull( pHashHeaders, "send_bytes" );
	const char* pReferer		= GetValueAvoidNull( pHashHeaders, "referer" );
	const char* pUserAgent		= GetValueAvoidNull( pHashHeaders, "user-agent" );
	const char* pAuthName		= GetValueAvoidNull( pHashHeaders, "authentication_user" );

	nRecordLen = STRLEN(pRemoteHost)
				+ STRLEN(pRequestLine)
				+ STRLEN(pReferer)
				+ STRLEN(pUserAgent)
				+ STRLEN(pAuthName);

	// ------------------------------------------------------------------------
	// create file
	// ------------------------------------------------------------------------
	EnterCriticalSection( pcs );		// <<<---

	hFile = CreateFile( pwszLogFile, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS, NULL);
	
	if( hFile == INVALID_HANDLE_VALUE )
	{
		LeaveCriticalSection( pcs );	// --->>>
		return 0;
	}

	SetFilePointer( hFile, 0, 0, FILE_END );

	if( nRecordLen < MAX_RECORD_LEN )
	{
		sprintf( szBuf, 
			"%s - %s [%02d/%s/%4d:%02d:%02d:%02d +0900] \"%s\" %d %s \"%s\" \"%s\"\n"
			, pRemoteHost
			, pAuthName
			, st.wDay, g_pszMonth[st.wMonth-1], st.wYear
			, st.wHour, st.wMinute, st.wSecond
			, pRequestLine
			, nHttpStatus
			, pSendBytes
			, pReferer
			, pUserAgent );
	}
	else
	{
		STRNCPY( szBuf, pRemoteHost, MAX_RECORD_LEN - 1 );
	}

	WriteFile( hFile, szBuf,STRLEN(szBuf), &dwWritten, NULL );

	DWORD dwFileSize = GetFileSize( hFile, NULL ); 

	// ------------------------------------------------------------------------
	// close file
	// ------------------------------------------------------------------------

	CloseHandle( hFile );
	hFile = INVALID_HANDLE_VALUE;

	// ------------------------------------------------------------------------
	// change backup if necessary
	// ------------------------------------------------------------------------

	if( dwFileSize > (DWORD)nLogSize )
	{
		int nLen = wcslen( pwszLogFile );
		WCHAR* pBackUp = (WCHAR*)malloc( nLen * sizeof(WCHAR) );
		
		if( pBackUp )
		{
			wcscpy( pBackUp, pwszLogFile );
			pBackUp[nLen-1] = '_';
			DeleteFile( pBackUp );
			MoveFile( pwszLogFile, pBackUp );
			free( pBackUp );
		}
	}

	// ------------------------------------------------------------------------
	LeaveCriticalSection( pcs );	// --->>>
	return 1;
}

static const char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey )
{
	char* p =  pHashHeaders->GetValueI( pszKey );
	if( p == NULL )
		return g_pNullString;
	return p;
}

static char *STRNCPY( char *dst, const char *src, size_t count )
{
	size_t ii;
	int flag;
	char* p = dst;

	for( ii = 0, flag = 0; ii < count; ii++ )
	{
		if( *src == '\0' )
			flag = 1;

		*dst++ = ( flag ? '\0' : *src++ );
	}

	return p;
}


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch( ul_reason_for_call )
	{
	case DLL_PROCESS_ATTACH:
		{
			if( pcs == NULL )
			{
				pcs = (CRITICAL_SECTION*)malloc( sizeof(CRITICAL_SECTION) );
				InitializeCriticalSection( pcs );
			}
			break;
		}
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		{
			if( !pcs )
			{
				DeleteCriticalSection( pcs );
				free( pcs );
				pcs = NULL;
			}
			break;
		}
    }

    return TRUE;
}

static unsigned int STRLEN( const char *string )
{
	if( string == NULL ) return 0;
	for( unsigned int ii = 0; *string != '\0'; ii++, string++ );
	return ii;
}
